
class ObjEvmToken:
    token_address = None
    abi = None
    chain_id = None 
    symbol = None
    name = None
    decimals = None
    token_class = None
    totalsupply = None
    keywords = None
    telegram_link = None
    creator_address = None
    creation_timestamp = None 
    creation_block_number = None 
    creation_tx_hash = None
    
    liquidity_pool_address = ''
    initialPrice = 0
    buy_gas_used_hp_check = 0 
    sell_gas_used_hp_check = 0 
    max_fee_per_gas = 100000000000
    max_prio_fee_per_gas = 3000000000
    gas_price = 100000000000
    slippage_percent = 50
    tax_buy_limit = 15
    tax_sell_limit = 60
    is_active = 'Y'
    is_completed = 'N'
    buy_with_hp_check = 'Y'
    created_at = 0
    modified_at = 0
    user_number = 0
    delayed_hp_protection = 'Y'
    automatic_profit_trading = 'Y'


    def __init__(self):
        self.db_id = 0

    def __eq__(self, other): 
        if not isinstance(other, ObjEvmToken):
            # don't attempt to compare against unrelated types
            return NotImplemented

        return (self.db_id == other.db_id and 
                self.txHash == other.txHash and
                self.network == other.network and
                self.chain_id == other.chain_id and
                self.tokenAddr == other.tokenAddr and
                self.liquidity_pool_address == other.liquidity_pool_address and
                self.amountBuy == other.amountBuy and
                self.amountToken == other.amountToken and
                self.token_symbol == other.token_symbol and
                self.profitTaken == other.profitTaken and
                self.initialPrice == other.initialPrice and
                self.buy_gas_used_hp_check == other.buy_gas_used_hp_check and
                self.sell_gas_used_hp_check == other.sell_gas_used_hp_check and
                self.max_fee_per_gas == other.max_fee_per_gas and
                self.max_prio_fee_per_gas == other.max_prio_fee_per_gas and
                self.gas_price == other.gas_price and
                self.slippage_percent == other.slippage_percent and
                self.tax_buy_limit == other.tax_buy_limit and
                self.tax_sell_limit == other.tax_sell_limit and
                self.is_active == other.is_active and
                self.is_completed == other.is_completed and
                self.buy_with_hp_check == other.buy_with_hp_check and
                self.created_at == other.created_at and
                self.modified_at == other.modified_at and
                self.user_number == other.user_number and
                self.delayed_hp_protection == other.delayed_hp_protection and
                self.automatic_profit_trading == other.automatic_profit_trading)

    def clone(self):
        retTok = ObjEvmToken()
        retTok.db_id = self.db_id
        retTok.txHash = self.txHash
        retTok.network = self.network
        retTok.chain_id = self.chain_id
        retTok.tokenAddr = self.tokenAddr
        retTok.liquidity_pool_address = self.liquidity_pool_address
        retTok.amountBuy = self.amountBuy
        retTok.amountToken = self.amountToken
        retTok.token_symbol = self.token_symbol
        retTok.profitTaken = self.profitTaken
        retTok.initialPrice = self.initialPrice
        retTok.buy_gas_used_hp_check = self.buy_gas_used_hp_check
        retTok.sell_gas_used_hp_check = self.sell_gas_used_hp_check
        retTok.max_fee_per_gas = self.max_fee_per_gas
        retTok.max_prio_fee_per_gas = self.max_prio_fee_per_gas
        retTok.gas_price = self.gas_price
        retTok.slippage_percent = self.slippage_percent
        retTok.tax_buy_limit = self.tax_buy_limit
        retTok.tax_sell_limit = self.tax_sell_limit
        retTok.is_active = self.is_active
        retTok.is_completed = self.is_completed
        retTok.buy_with_hp_check = self.buy_with_hp_check
        retTok.created_at = self.created_at
        retTok.modified_at = self.modified_at
        retTok.user_number = self.user_number
        retTok.delayed_hp_protection = self.delayed_hp_protection
        retTok.automatic_profit_trading = self.automatic_profit_trading
        return retTok


from psycopg2 import sql
from ..DataStructures import ObjEvmToken
from ..rupine_db import herokuDbAccess

def postEvmToken(connection, schema:str, token:ObjEvmToken):

    query = sql.SQL("INSERT INTO {}.evm_token (token_address, abi, chain_id, symbol, name, decimals, token_class, totalsupply, keywords, telegram_link, creator_address, creation_timestamp, creation_block_number, creation_tx_hash) \
           VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        token.token_address,
        token.abi,
        token.chain_id,
        token.symbol,
        token.name,
        token.decimals,
        token.token_class,
        token.totalsupply,
        token.keywords,
        token.telegram_link,
        token.creator_address,
        token.creation_timestamp,
        token.creation_block_number,
        token.creation_tx_hash)
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getEvmToken(connection, schema, chain_id, token_address):
    
    query = sql.SQL("SELECT * FROM {}.evm_token WHERE chain_id = %s AND token_address = %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,token_address], connection)    
    return result

def getEvmTokenWithoutName(connection, schema, chain_id, gteCreatedAt):
    
    query = sql.SQL("SELECT * FROM {}.evm_token WHERE chain_id = %s AND name is NULL AND created_at >= %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,gteCreatedAt], connection)    
    return result


from ..rupine_db import herokuDbAccess
from psycopg2 import sql

def getEvmLatestTokenLiquidityPools(connection, schema, chain_id):
    
    query = sql.SQL("SELECT * FROM {}.v_evm_latest_token_liquidity_pools WHERE chain_id = %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id], connection)    
    return result

def getEvmLatestTokenLiquidityPoolsWithoutName(connection, schema, chain_id, gteCreatedAt):
    
    query = sql.SQL("SELECT * FROM {}.v_evm_latest_token_liquidity_pools WHERE chain_id = %s AND name = 'n/a' AND created_at >= %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,gteCreatedAt], connection)    
    return result

def getEvmLatestTokenLiquidityPoolsNotLaunched(connection, schema, chain_id, gteCreatedAt):
    
    query = sql.SQL("SELECT * FROM {}.v_evm_latest_token_liquidity_pools WHERE chain_id = %s AND name = 'n/a' AND created_at >= %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,gteCreatedAt], connection)    
    return result


from RupineHeroku.rupine_db import herokuDbAccess
from psycopg2 import sql

def postTaxTransaction(connection, schema, data):

    query = sql.SQL("INSERT INTO {}.tax_transaction (chain_id,chain,public_address,timestamp,block_number,transaction_hash,category,token,amount,usd_value,eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        data['chain_id'],
        data['chain'],
        data['public_address'],
        data['timestamp'],
        data['block_number'],
        data['transaction_hash'],
        data['category'],
        data['token'],
        data['amount'],
        data['usd_value'],
        data['eur_value'])
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getTaxTransaction(connection, schema, token:str=None, timestamp:int=0, posNegAll:str='all'):
    '''
    Parameters:
        - token: String of Token, e.g. URHT, DUSD-URTH, etc. Default is None
        - timestamp: all data with timestamp gte. Default is 0
        - posNegAll: "positive", "negative" or "all". is Amount gte 0, lt 0 or everything. Default is 'all'
    '''
    conditions = ""
    params = []
    if token != None:
        conditions = conditions + " AND t.token = %s"
        params.append(token)
    
    if posNegAll == 'positive':
        conditions = conditions + " AND t.amount >= 0"
    elif posNegAll == 'negative':
        conditions = conditions + " AND t.amount < 0"

    order = " ORDER BY t.timestamp ASC, c.sort_no ASC"
 
    query = sql.SQL("SELECT t.*,CASE WHEN c.sort_no IS NULL THEN '999' ELSE c.sort_no END AS sort_no FROM {0}.tax_transaction t LEFT JOIN {0}.tax_category c \
        ON t.category = c.category \
        WHERE 1=1 AND t.timestamp >= %s" + conditions + order).format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [timestamp,*params], connection)    
       
    return result


def postTaxReward(connection, schema, data):

    query = sql.SQL("INSERT INTO {}.tax_reward (chain_id,chain,public_address,timestamp,category,token,amount,usd_value,eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
        
    params = (
        data['chain_id'],
        data['chain'],
        data['public_address'],
        data['timestamp'],
        data['category'],
        data['token'],
        data['amount'],
        data['usd_value'],
        data['eur_value']
    )

    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result